/*******************************************************************************
*                                                                              *
* System      : Xilinx Spartan-6 Starter Kit (SP601)                           *
* Block       : ----                                                           *
* Version     : v 0.1.0 2010/03/31                                             *
*               v 0.2.0 2010/04/21 Change polarity of system reset             *
*               v 0.2.1 2010/05/10 Add GMII_1000M                              *
*               v 0.2.2 2010/12/14 SiTCP FIFO size reduced to 16kB             *
*                                                                              *
* Description : The Network Processor (SiTCP)                                  *
*               The standard implementation                                    *
*                                                                              *
*                Copyright (c) 2010 Bee Beans Technologies Co.,Ltd.            *
*                All rights reserved                                           *
*                                                                              *
*******************************************************************************/
`define FPGA_VER 32'h1005_1001
`define FPGA_ID  32'h0209_0018

`define REG_INIT_X08 8'h08
`define REG_INIT_X0A 8'h0A
`define REG_INIT_X0B 8'h0B
`define REG_INIT_X0C 8'h0C
`define REG_INIT_X0D 8'h0D
`define REG_INIT_X0E 8'h0E
`define REG_INIT_X0F 8'h0F

`include "../CoreGen/sys_dcm.v"
`include "../CoreGen/gmii_dcm.v"
`include "../CoreGen/sitcp_fifo.v"
`include "../SiTCP/SiTCP_XC6S_16K_BBT_V40.V"
`include "../SiTCP/WRAP_SiTCP_GMII_XC6S_16K.V"
`include "../SiTCP/TIMER.V"
`include "../Src/RBCP_REG.V"

module XC6S_SP601_SiTCP(
// System
	OSC					,	// in	: 27MHz clock
	DIFF_CLK_P			,	// in	: 200MHz
	DIFF_CLK_N			,	// in	: 200MHz
	SW_RST				,	// in	: System reset
// GMII
	GMII_RSTn			,	// out	: Reset
	GMII_GTXCLK			,	// out	: GbE Tx Clock
	GMII_TX_CLK			,	// in	: Tx clock
	GMII_TX_EN			,	// out	: Tx enable
	GMII_TXD			,	// out	: Tx data[3:0]
	GMII_TX_ER			,	// out	: TX error
	// RX
	GMII_RX_CLK			,	// in	: Rx clock
	GMII_RX_DV			,	// in	: Rx data valid
	GMII_RXD			,	// in	: Rx data[3:0]
	GMII_RX_ER			,	// in	: Rx error
	GMII_CRS			,	// in	: Carrier sense
	GMII_COL			,	// in	: Collision detected
	// Management IF
	GMII_MDC			,	// out	: Clock for MDIO
	GMII_MDIO			,	// io	: Data
// SW, LED
	DIP_SW				,	// in	: SW[3:0]
	LED						// out	: LED[3:0]
);

//-------- Input/Output -------------
	input			OSC				;
	input			DIFF_CLK_P		;
	input			DIFF_CLK_N		;
	input			SW_RST			;

	output			GMII_RSTn		;
	output			GMII_GTXCLK		;
	input			GMII_TX_CLK		;
	output			GMII_TX_EN		;
	output	[7:0]	GMII_TXD		;
	output			GMII_TX_ER		;
	// RX
	input			GMII_RX_CLK		;
	input			GMII_RX_DV		;
	input	[7:0]	GMII_RXD		;
	input			GMII_RX_ER		;
	input			GMII_CRS		;
	input			GMII_COL		;
	// Management IF
	output			GMII_MDC		;
	inout			GMII_MDIO		;

	input	[3:0]	DIP_SW			;
	output	[3:0]	LED				;

//------------------------------------------------------------------------------
//	Buffers
//------------------------------------------------------------------------------
	assign	GMII_MDIO	= (GMII_MDIO_OE	? GMII_MDIO_OUT	: 1'bz);

//------------------------------------------------------------------------------
//	PLL
//------------------------------------------------------------------------------
	wire			sysDcmRst			;
	wire			sysDcmLocked		;
	wire			SYSCLK				;

	assign	sysDcmRst		= SW_RST;

	wire			DIFF_CLK			;

	IBUFGDS IBUFGDS_SYSCLK (.O(DIFF_CLK), .I(DIFF_CLK_P), .IB(DIFF_CLK_N));

   	sys_dcm			SYSDCM(
	    .CLK_IN1		(DIFF_CLK		),
		.RESET			(sysDcmRst		),
		.CLK_OUT1		(SYSCLK			),	// 130MHz
		.LOCKED			(sysDcmLocked	)
	);

	wire			gmiiDcmRst			;
	wire			gmiiDcmLocked		;
	wire			CLK125M				;

	assign	gmiiDcmRst		= SW_RST | ~sysDcmLocked;

   	gmii_dcm			GMIIDCM(
		.CLK_IN1		(DIFF_CLK		),
		.RESET			(gmiiDcmRst		),
		.CLK_OUT1		(CLK125M		),
		.LOCKED			(gmiiDcmLocked	)
	);

	assign	RST	= ~sysDcmLocked | ~gmiiDcmLocked | sysDcmRst;

	wire	GMII_1000M		;
	
	assign	GMII_1000M = DIP_SW[3];


	BUFGMUX GMIIMUX(.O(int_GMII_TX_CLK), .I0(GMII_TX_CLK), .I1(CLK125M), .S(GMII_1000M));

	ODDR2	IOB_GTX		(.Q(GMII_GTXCLK), .C0(CLK125M), .C1(~CLK125M), .CE(1'b1), .D0(1'b1), .D1(1'b0), .R(1'b0), .S(1'b0));

//------------------------------------------------------------------------------
//	LED
//------------------------------------------------------------------------------
	reg	[23:0]	ledCnt;

	always@ (posedge SYSCLK or posedge RST) begin
		if(RST)begin
			ledCnt[23:0] <= 24'd0;
		end else begin
			ledCnt[23:0] <= ledCnt[23:0] + 24'd1;
		end
	end

	assign	LED[3]	= ledCnt[23] & ledCnt[22];
//	assign	LED[2]	= DIP_SW[2];
//	assign	LED[1]	= DIP_SW[1];
//	assign	LED[0]	= DIP_SW[0];

//------------------------------------------------------------------------------
//	NETWORK PROTOCOL PROCESSOR
//------------------------------------------------------------------------------
	wire	[15:0]	TCP_RX_WC			;
	wire	[7:0]	TCP_RX_DATA			;
	wire	[7:0]	TCP_TX_DATA			;

	wire	[31:0]	RBCP_ADDR			;
	wire	[7:0]	RBCP_WD				;
	wire	[7:0]	RBCP_RD				;

	WRAP_SiTCP_GMII_XC6S_16K		
		#(130) // = System clock frequency(MHz), integer only
	SiTCP(
		.CLK					(SYSCLK				),	// in	: System Clock > 129MHz
		.RST					(RST				),	// in	: System reset
	// Configuration parameters
		.FORCE_DEFAULTn			(1'b0				),	// in	: Load default parameters
		.EXT_IP_ADDR			(32'd0				),	// in	: IP address[31:0]
		.EXT_TCP_PORT			(16'd0				),	// in	: TCP port #[15:0]
		.EXT_RBCP_PORT			(16'd0				),	// in	: RBCP port #[15:0]
		.PHY_ADDR				(5'b00111			),	// in	: PHY-device MIF address[4:0]
	// EEPROM
		.EEPROM_CS				(),	// out	: Chip select
		.EEPROM_SK				(),	// out	: Serial data clock
		.EEPROM_DI				(),	// out	: Serial write data
		.EEPROM_DO				(1'b0),	// in	: Serial read data
	// MII interface
		.GMII_RSTn				(GMII_RSTn			),	// out	: PHY reset
		.GMII_1000M				(GMII_1000M			),	// in	: GMII mode (0:MII, 1:GMII)
		// TX
		.GMII_TX_CLK			(int_GMII_TX_CLK	),	// in	: Tx clock
		.GMII_TX_EN				(GMII_TX_EN			),	// out	: Tx enable
		.GMII_TXD				(GMII_TXD[7:0]		),	// out	: Tx data[7:0]
		.GMII_TX_ER				(GMII_TX_ER			),	// out	: TX error
		// RX
		.GMII_RX_CLK			(GMII_RX_CLK		),	// in	: Rx clock
		.GMII_RX_DV				(GMII_RX_DV			),	// in	: Rx data valid
		.GMII_RXD				(GMII_RXD[7:0]		),	// in	: Rx data[7:0]
		.GMII_RX_ER				(GMII_RX_ER			),	// in	: Rx error
		.GMII_CRS				(GMII_CRS			),	// in	: Carrier sense
		.GMII_COL				(GMII_COL			),	// in	: Collision detected
		// Management IF
		.GMII_MDC				(GMII_MDC			),	// out	: Clock for MDIO
		.GMII_MDIO_IN			(GMII_MDIO			),	// in	: Data
		.GMII_MDIO_OUT			(GMII_MDIO_OUT		),	// out	: Data
		.GMII_MDIO_OE			(GMII_MDIO_OE		),	// out	: MDIO output enable
	// User I/F
		.SiTCP_RST				(SiTCP_RST			),	// out	: Reset for SiTCP and related circuits
		// TCP connection control
		.TCP_OPEN_REQ			(1'b0				),	// in	: Reserved input, shoud be 0
		.TCP_OPEN_ACK			(TCP_OPEN			),	// out	: Acknowledge for open (=Socket busy)
		.TCP_ERROR				(),	// out	: TCP error, its active period is equal to MSL
		.TCP_CLOSE_REQ			(TCP_CLOSE			),	// out	: Connection close request
		.TCP_CLOSE_ACK			(TCP_CLOSE			),	// in	: Acknowledge for closing
		// FIFO I/F
		.TCP_RX_WC				(TCP_RX_WC[15:0]	),	// in	: Rx FIFO write count[15:0] (Unused bits should be set 1)
		.TCP_RX_WR				(TCP_RX_WR			),	// out	: Write enable
		.TCP_RX_DATA			(TCP_RX_DATA[7:0]	),	// out	: Write data[7:0]
		.TCP_TX_FULL			(TCP_TX_FULL		),	// out	: Almost full flag
		.TCP_TX_WR				(TCP_TX_WR			),	// in	: Write enable
		.TCP_TX_DATA			(TCP_TX_DATA[7:0]	),	// in	: Write data[7:0]
		// RBCP
		.RBCP_ACT				(RBCP_ACT			),	// out	: RBCP active
		.RBCP_ADDR				(RBCP_ADDR[31:0]	),	// out	: Address[31:0]
		.RBCP_WD				(RBCP_WD[7:0]		),	// out	: Data[7:0]
		.RBCP_WE				(RBCP_WE			),	// out	: Write enable
		.RBCP_RE				(RBCP_RE			),	// out	: Read enable
		.RBCP_ACK				(RBCP_ACK			),	// in	: Access acknowledge
		.RBCP_RD				(RBCP_RD[7:0]		)	// in	: Read data[7:0]
	);

//------------------------------------------------------------------------------
//	TCP loopback FIFOl
//------------------------------------------------------------------------------

	assign	TCP_RX_WC[15:11]	= 5'b11111;

	sitcp_fifo		sitcp_fifo(
		.clk			(SYSCLK				),
		.rst			(TCP_OPEN			),
		.data_count		(TCP_RX_WC[10:0]	),
		.full			(),
		.wr_en			(TCP_RX_WR			),
		.din			(TCP_RX_DATA[7:0]	),
		.empty			(sitcpFifoEmpty		),
		.rd_en			(sitcpFifoRe		),
		.dout			(TCP_TX_DATA[7:0]	),
		.valid			(TCP_TX_WR			)
	);

	assign	sitcpFifoRe	= ~TCP_TX_FULL & ~sitcpFifoEmpty;

//------------------------------------------------------------------------------
//	Registers
//------------------------------------------------------------------------------
	RBCP_REG		RBCP_REG(
		// System
		.CLK					(SYSCLK				),	// in	: System clock
		.RST					(SiTCP_RST			),	// in	: System reset
		.LED					(LED[2:0]			),	// out	: LED[2:0]
		.DIP_SW					(DIP_SW[3:0]		),	// in	: DIP_SW[3:0]
		// RBCP I/F
		.RBCP_ACT				(RBCP_ACT			),	// in	: Active
		.RBCP_ADDR				(RBCP_ADDR[31:0]	),	// in	: Address[31:0]
		.RBCP_WE				(RBCP_WE			),	// in	: Write enable
		.RBCP_WD				(RBCP_WD[7:0]		),	// in	: Write data[7:0]
		.RBCP_RE				(RBCP_RE			),	// in	: Read enable
		.RBCP_RD				(RBCP_RD[7:0]		),	// out	: Read data[7:0]
		.RBCP_ACK				(RBCP_ACK			)	// out	: Acknowledge
	);

//------------------------------------------------------------------------------
endmodule